import yaml
import click
import pandas as pd
from typing import Type

from ankimaker.config import Config


__CONFIRMATION_QUESTION = """
Is this read option {option} correct?
______________
{preview}
"""

__SUCCESS_MESSAGE = """
All done. You can now use your configuration with this  command:
{command}
"""

__COMMAND_SAMPLE = """ankimaker csv \
-i {input} \
-o my-new-deck.apkg \
--conf {output}
"""

def create_config(input_file, output_path):
    new_config = Config()

    new_config.separators = handle_read_option(
        input_file, read_option='sep', sep=new_config.separators
    )
    new_config.header = handle_read_option(
        input_file, read_option='header', header=new_config.header,
        sep=new_config.separators, option_type=int
    )
    finish_message = __SUCCESS_MESSAGE.format(command=make_sample_command(input_file, output_path))
    click.echo(finish_message)


def handle_read_option(input_file, read_option, option_type: Type = str, **kargs):
    preview: str
    is_finished = False
    while not is_finished:
        preview = load_preview(input_file, **kargs)
        question = __CONFIRMATION_QUESTION.format(
            option=str(read_option),
            preview=preview
        )
        answer: str = click.prompt(question, type=str, default='y')
        click.clear()
        if answer.lower()[0] == 'y':
            return kargs[read_option]
        else:
            updated_option: str = click.prompt('Insert the correct', type=option_type)
            kargs[read_option] = updated_option
            click.clear()


def load_preview(input_file, *args, **kargs):
    try:
        df = pd.read_csv(input_file, *args, **kargs)
        preview = df.head()
    except pd.errors.ParserError:
        preview = 'FAIL'
    return preview


def save_file(config: Config, file_path):
    f = open(file_path, 'w')
    yaml.dump(config, f)


def make_sample_command(inputf, output):
    command = __COMMAND_SAMPLE.format(
        input=inputf, output=output
    )
    return command
